#!/bin/bash 

# Copyright (C) 2007-2020 Centrify Corporation. All rights reserved.
#
# Collect all necessary files for adinfo --support
# Store in /var/centrify/tmp/adinfo_support.tar and zipped or compressed.
#

#
# Parameter:
# $* - path of the adinfo_support.txt is in the end of additional paths

# Set Paths
CIMS_USR=/usr
CIMS_INSTALL=/usr/share/centrifydc

OSNAME=`uname -s`
ADDITIONAL_PATHS="$*" # Get additional paths
DEST=""
REG_USERS_TMPDIR="/var/centrify/tmp/users_gp.report" # store users' gp.report directory
LOG_DIR=""
if [ $OSNAME = "HP-UX" ]; then
    LOG_DIR="/var/adm/syslog";
else
    LOG_DIR="/var/log/";
fi
SOURCE="/var/log/centrify_client.log \
        /var/log/centrifydc-install.log \
        /var/adm/syslog/centrify_client.log \
        /var/adm/syslog/centrifydc-install.log \
        /etc/*release \
        /etc/centrifydc \
        /etc/ssh \
        /opt/ssh/etc \
        /etc/ssh_* \
        /etc/sshd_* \
        /usr/local/etc/ssh_* \
        /usr/local/etc/sshd_* \
        /var/centrifydc/httpproxy.cred \
        /var/centrifydc/*.cache \
        /var/centrifydc/*.idx \
        /var/centrifydc/nis/* \
        /var/centrifydc/reg/machine/gp.report \
        /var/centrifydc/kset.* \
        /var/centrify/tmp/dumped_cache \
        /var/centrify/tmp/dump_cache_error.log \
        /var/centrify/tmp/stacktrace.txt \
        /var/centrify/tmp/adcdiagCheckingReport*.log \
        /var/centrify/tmp/ldapproxy.cache.out* \
        /var/centrifyda/tmp/support.tar.gz \
        /var/centrifyda/tmp/dainfo_support.tar.gz \
        /var/log/audit/audit.log \
        /var/log/audit/audit.log.*
        $LOG_DIR/centrifydc.log.* \
        $REG_USERS_TMPDIR $ADDITIONAL_PATHS"

if [ $OSNAME = "Darwin" ]; then
    SOURCE="$SOURCE \
            /var/log/centrifydc-login.log* \
            /var/log/system.log"
elif [ $OSNAME = "SunOS" -o $OSNAME = "Linux" ]; then
    $CIMS_INSTALL/bin/nscdrestart.sh status 2>&1 > /dev/null
    if [ $? -eq 0 ]; then
        SOURCE="$SOURCE \
            /etc/nscd.conf"
    fi
fi

REG_USERS_SOURCEDIR="/var/centrifydc/reg/users"
REG_USERS_TMPDIR_BAK="/var/centrify/tmp/users_gp.report.bak.$$"

TARGET_OS=""
CMD_AWK="awk"
CMD_SED="sed"
CDCSTATE=`$CIMS_USR/bin/adinfo -m`

identify_os()
{
    UNAME=`uname -a`
    case "$UNAME" in
        SunOS*)
            TARGET_OS="solaris"
            CMD_AWK="nawk"
            ;;
        *coreos*)
            TARGET_OS="coreos"
            ;;
    esac
}

# Get destination directory by the path of the adinfo_support.txt
# because the path of the adinfo_support.txt is in the end of path
get_dest()
{
    SUPPORTFILE=`echo $ADDITIONAL_PATHS | ${CMD_AWK} '{print $NF}'`
    DESTDIR=`echo $SUPPORTFILE|sed 's/[^/]*$//g'`
    DEST="${DESTDIR}adinfo_support.tar"
}

# maybe have many users, so copy users' gp.report files for storing adinfo_support.tar with command tar
get_users_gpfiles()
{
    if [ -d "${REG_USERS_SOURCEDIR}" ]; then
        # back up REG_USERS_TMPDIR if have
        if [ -d "${REG_USERS_TMPDIR}" ]; then
            mv "${REG_USERS_TMPDIR}" "${REG_USERS_TMPDIR_BAK}"
        fi
        mkdir -p "${REG_USERS_TMPDIR}"
        find "${REG_USERS_SOURCEDIR}" -name "gp.report" 2> /dev/null | while read line; do
            username=`echo "${line}" | ${CMD_AWK} -F "/" '{print $(NF-1)}'`
            mkdir "${REG_USERS_TMPDIR}/${username}"
            cp "${line}" "${REG_USERS_TMPDIR}/${username}/"
        done
    fi
}

# the cdc perl modules/scripts got changed. 
#     adinfo_pkgverify.sh will show only changed/missing,
#     here we will just collect then gzip them, 
#     the change details (size/perm/checksum, etc) can be 
#     got from the file combined with 
#     the previous called 'adinfo_pkgverify.sh list' in 'adinfo -t'
get_pl_pm()
{
    verify=`$CIMS_INSTALL/bin/adinfo_pkgverify.sh` 
    if [ $? -ne 0 ]; then
        PL_PM=`echo "$verify" \
            | ${CMD_AWK} '{print $2}' \
            | sed -n 's/\(.*\.p[lm]\)$/\1/p' `
    fi 
    SOURCE="$SOURCE $PL_PM"
}

cleanup_tmpdir()
{
    if [ -d "${REG_USERS_TMPDIR_BAK}" ]; then
        mv "${REG_USERS_TMPDIR_BAK}" "${REG_USERS_TMPDIR}"
    fi
    rm -rf "${REG_USERS_TMPDIR}"
}

# main
umask 0277
identify_os

if [ "$TARGET_OS" = "coreos" ]; then
    # get compressed core dump files generated by adclient, filter out the ones that are inaccessible
    CORE_LIST=`coredumpctl info adclient | grep "Storage: /" | grep -v inaccessible | cut -d: -f2 2>/dev/null`
    SOURCE="$SOURCE \
            $CORE_LIST"
fi

get_users_gpfiles
get_dest
get_pl_pm

if [ "$DESTDIR" = "/var/centrify/tmp/" ]; then
    DESTDIR="`echo get_temp_dir | $CIMS_USR/bin/adedit`/"
fi
if [ -f $DEST ]; then
    echo "Remove previous $DEST"
    rm -f $DEST
fi
if [ -f $DEST.gz ]; then
    echo "Remove previous $DEST.gz"
    rm -f $DEST.gz
fi
if [ -f $DEST.Z ]; then
    echo "Remove previous $DEST.Z"
    rm -f $DEST.Z
fi
# For DC, it does not know which version of DA is installed 
# therefore the DA support file could be either 
# "dainfo_support.tar.gz" OR "support.tar.gz".
if [ -f /var/centrifyda/tmp/dainfo_support.tar.gz ]; then
    echo "Remove previous /var/centrifyda/tmp/dainfo_support.tar.gz"
    rm -f /var/centrifyda/tmp/dainfo_support.tar.gz
fi
if [ -f /var/centrifyda/tmp/support.tar.gz ]; then
    echo "Remove previous /var/centrifyda/tmp/support.tar.gz"
    rm -f /var/centrifyda/tmp/support.tar.gz
fi
if [ "$CDCSTATE" = "down" ]; then
    $CIMS_INSTALL/bin/adcache -o /var/centrify/tmp/dumped_cache 2>/var/centrify/tmp/dump_cache_error.log
else
    $CIMS_INSTALL/bin/adcache -L -o /var/centrify/tmp/dumped_cache 2>/var/centrify/tmp/dump_cache_error.log
fi
# Collect information for local accounts
$CIMS_INSTALL/bin/admanagelocal --status > /var/centrify/tmp/local_account.log 2>&1
if [ $? -eq 156 ]; then
    echo "" >> /var/centrify/tmp/local_account.log
    echo "All managed local users:" >> /var/centrify/tmp/local_account.log
    $CIMS_INSTALL/bin/admanagelocal --dump user >> /var/centrify/tmp/local_account.log 2>&1
    echo "" >> /var/centrify/tmp/local_account.log
    echo "All managed local groups:" >> /var/centrify/tmp/local_account.log
    $CIMS_INSTALL/bin/admanagelocal --dump group >> /var/centrify/tmp/local_account.log 2>&1
fi

rm -f /var/centrify/tmp/adcdiagCheckingReport*.log > /dev/null 2>&1
$CIMS_INSTALL/bin/adcdiag 2>&1 > /dev/null

rm -f /var/centrify/tmp/ldapproxy.cache.out* > /dev/null 2>&1
RUN_RESULT="`$CIMS_INSTALL/bin/centrify-ldapproxy status 2> /dev/null`"
SLAPD_PID=`echo "$RUN_RESULT" | grep pid | ${CMD_SED} -e 's/^.*(pid //' -e 's/).*$//'`
[ -n "$SLAPD_PID" ] && /bin/kill -SIGUSR1 $SLAPD_PID

if [ -x "/usr/bin/dainfo" ]; then
    echo ""
    echo "Collecting DA support information via \"dainfo -t\":"
    /usr/bin/dainfo -t
fi
echo "Collecting information for adinfo --support now..."
# skip directory and any file, which name end with key, 
# e.g ssh private key
# Note: the max length of command line is OS dependent, e.g. HPUX ARG_MAX is 2M , while RHEL seems like just 131K. 
# So, we may also need to check the length of the arg, when the number of files to pack increases. 
SOURCE_TO_PACK=`
find $SOURCE 2>/dev/null | while read line
do
    test -d $line 2>/dev/null \
    || ( echo $line | grep -v "key$" > /dev/null \
    && echo $line )
done`
tar -cf $DEST $SOURCE_TO_PACK 2>/dev/null

cleanup_tmpdir

if [ -f $DEST ]; then
    #Like adinfo_support.txt file, only Allow root read right too.
    chmod 400 $DEST
    
    echo "Collection finished. Compress now."
    gzip $DEST

    if [ $? -eq 0 ]; then
        echo "Successfully gzip $DEST."
    else
        echo "gzip failed, using compress instead."
        compress $DEST
        
        if [ $? -eq 0 ]; then
            echo "Successfully compress $DEST."
        else 
            echo "compress failed. Keep $DEST."
        fi
    fi
else
    echo "Collection failed!"
fi
